@@ -128,7 +128,7 @@ nautilus_drag_uri_array_from_selection_list (const GList *selection_list)
 
 	uri_list = nautilus_drag_uri_list_from_selection_list (selection_list);
 	uris = nautilus_drag_uri_array_from_list (uri_list);
-	eel_g_list_free_deep (uri_list);
+	g_list_free_full (uri_list, g_free);
 
 	return uris;
 }
@@ -198,8 +198,8 @@ nautilus_drag_build_selection_list (GtkSelectionData *data)
 	int size;
 
 	result = NULL;
-	oldp = data->data;
-	size = data->length;
+	oldp = gtk_selection_data_get_data (data);
+	size = gtk_selection_data_get_length (data);
 
 	while (size > 0) {
 		NautilusDragSelectionItem *item;
@@ -375,15 +375,15 @@ nautilus_drag_default_drop_action_for_netscape_url (GdkDragContext *context)
 {
 	/* Mozilla defaults to copy, but unless thats the
 	   only allowed thing (enforced by ctrl) we want to ASK */
-	if (context->suggested_action == GDK_ACTION_COPY &&
-	    context->actions != GDK_ACTION_COPY) {
+	if (gdk_drag_context_get_suggested_action (context) == GDK_ACTION_COPY &&
+	    gdk_drag_context_get_actions (context) != GDK_ACTION_COPY) {
 		return GDK_ACTION_ASK;
-	} else if (context->suggested_action == GDK_ACTION_MOVE) {
+	} else if (gdk_drag_context_get_suggested_action (context) == GDK_ACTION_MOVE) {
 		/* Don't support move */
 		return GDK_ACTION_COPY;
 	}
 	
-	return context->suggested_action;
+	return gdk_drag_context_get_suggested_action (context);
 }
 
 static gboolean
@@ -446,16 +446,16 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		return;
 	}
 
-	actions = context->actions & (GDK_ACTION_MOVE | GDK_ACTION_COPY);
+	actions = gdk_drag_context_get_actions (context) & (GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	if (actions == 0) {
 		 /* We can't use copy or move, just go with the suggested action. */
-		*action = context->suggested_action;
+		*action = gdk_drag_context_get_suggested_action (context);
 		return;
 	}
 
-	if (context->suggested_action == GDK_ACTION_ASK) {
+	if (gdk_drag_context_get_suggested_action (context) == GDK_ACTION_ASK) {
 		/* Don't override ask */
-		*action = context->suggested_action;
+		*action = gdk_drag_context_get_suggested_action (context);
 		return;
 	}
 	
@@ -536,13 +536,13 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		} else {
-			*action = context->suggested_action;
+			*action = gdk_drag_context_get_suggested_action (context);
 		}
 	} else {
 		if (actions & GDK_ACTION_COPY) {
 			*action = GDK_ACTION_COPY;
 		} else {
-			*action = context->suggested_action;
+			*action = gdk_drag_context_get_suggested_action (context);
 		}
 	}
 
@@ -555,11 +555,11 @@ GdkDragAction
 nautilus_drag_default_drop_action_for_uri_list (GdkDragContext *context,
 						const char *target_uri_string)
 {
-	if (eel_uri_is_trash (target_uri_string) && (context->actions & GDK_ACTION_MOVE)) {
+	if (eel_uri_is_trash (target_uri_string) && (gdk_drag_context_get_actions (context) & GDK_ACTION_MOVE)) {
 		/* Only move to Trash */
 		return GDK_ACTION_MOVE;
 	} else {
-		return context->suggested_action;
+		return gdk_drag_context_get_suggested_action (context);
 	}
 }
 
@@ -694,7 +694,7 @@ nautilus_drag_drag_data_get (GtkWidget *widget,
 	}
 	
 	gtk_selection_data_set (selection_data,
-				selection_data->target,
+				gtk_selection_data_get_target (selection_data),
 				8, result->str, result->len);
 	g_string_free (result, TRUE);
 
@@ -816,65 +816,13 @@ nautilus_drag_drop_action_ask (GtkWidget *widget,
 	gtk_grab_remove (menu);
 	
 	g_main_loop_unref (damd.loop);
-	
-	gtk_object_sink (GTK_OBJECT (menu));
-	
+
+	g_object_ref_sink (menu);
+	g_object_unref (menu);
+
 	return damd.chosen;
 }
 
-GdkDragAction
-nautilus_drag_drop_background_ask (GtkWidget *widget, 
-				   GdkDragAction actions)
-{
-	GtkWidget *menu;
-	GtkWidget *menu_item;
-	DropActionMenuData damd;
-	
-	/* Create the menu and set the sensitivity of the items based on the
-	 * allowed actions.
-	 */
-	menu = gtk_menu_new ();
-	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (widget));
-	
-	append_drop_action_menu_item (menu, _("Set as background for _all folders"),
-				      NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND,
-				      (actions & NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND) != 0,
-				      &damd);
-
-	append_drop_action_menu_item (menu, _("Set as background for _this folder"),
-				      NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND,
-				      (actions & NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND) != 0,
-				      &damd);
-
-	eel_gtk_menu_append_separator (GTK_MENU (menu));
-	
-	menu_item = gtk_menu_item_new_with_mnemonic (_("Cancel"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-	gtk_widget_show (menu_item);
-	
-	damd.chosen = 0;
-	damd.loop = g_main_loop_new (NULL, FALSE);
-
-	g_signal_connect (menu, "deactivate",
-			  G_CALLBACK (menu_deactivate_callback),
-			  &damd);
-	
-	gtk_grab_add (menu);
-	
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-			NULL, NULL, 0, GDK_CURRENT_TIME);
-	
-	g_main_loop_run (damd.loop);
-
-	gtk_grab_remove (menu);
-	
-	g_main_loop_unref (damd.loop);
-	
-	gtk_object_sink (GTK_OBJECT (menu));
-	
-	return damd.chosen;	
-}
-
 gboolean
 nautilus_drag_autoscroll_in_scroll_region (GtkWidget *widget)
 {
@@ -889,11 +837,12 @@ nautilus_drag_autoscroll_in_scroll_region (GtkWidget *widget)
 void
 nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_delta, float *y_scroll_delta)
 {
+	GtkAllocation allocation;
 	int x, y;
 
 	g_assert (GTK_IS_WIDGET (widget));
 
-	gdk_window_get_pointer (widget->window, &x, &y, NULL);
+	gdk_window_get_pointer (gtk_widget_get_window (widget), &x, &y, NULL);
 
 	/* Find out if we are anywhere close to the tree view edges
 	 * to see if we need to autoscroll.
@@ -905,7 +854,8 @@ nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_del
 		*x_scroll_delta = (float)(x - AUTO_SCROLL_MARGIN);
 	}
 
-	if (x > widget->allocation.width - AUTO_SCROLL_MARGIN) {
+	gtk_widget_get_allocation (widget, &allocation);
+	if (x > allocation.width - AUTO_SCROLL_MARGIN) {
 		if (*x_scroll_delta != 0) {
 			/* Already trying to scroll because of being too close to 
 			 * the top edge -- must be the window is really short,
@@ -913,14 +863,14 @@ nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_del
 			 */
 			return;
 		}
-		*x_scroll_delta = (float)(x - (widget->allocation.width - AUTO_SCROLL_MARGIN));
+		*x_scroll_delta = (float)(x - (allocation.width - AUTO_SCROLL_MARGIN));
 	}
 
 	if (y < AUTO_SCROLL_MARGIN) {
 		*y_scroll_delta = (float)(y - AUTO_SCROLL_MARGIN);
 	}
 
-	if (y > widget->allocation.height - AUTO_SCROLL_MARGIN) {
+	if (y > allocation.height - AUTO_SCROLL_MARGIN) {
 		if (*y_scroll_delta != 0) {
 			/* Already trying to scroll because of being too close to 
 			 * the top edge -- must be the window is really narrow,
@@ -928,7 +878,7 @@ nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_del
 			 */
 			return;
 		}
-		*y_scroll_delta = (float)(y - (widget->allocation.height - AUTO_SCROLL_MARGIN));
+		*y_scroll_delta = (float)(y - (allocation.height - AUTO_SCROLL_MARGIN));
 	}
 
 	if (*x_scroll_delta == 0 && *y_scroll_delta == 0) {
@@ -1008,7 +958,7 @@ nautilus_drag_selection_includes_special_link (GList *selection_list)
 	return FALSE;
 }
 
-static void
+static gboolean
 slot_proxy_drag_motion (GtkWidget          *widget,
 			GdkDragContext     *context,
 			int                 x,
@@ -1036,6 +986,11 @@ slot_proxy_drag_motion (GtkWidget          *widget,
 
 	if (!drag_info->have_data) {
 		target = gtk_drag_dest_find_target (widget, context, NULL);
+
+		if (target == GDK_NONE) {
+			goto out;
+		}
+
 		gtk_drag_get_data (widget, context, target, time);
 	}
 
@@ -1077,6 +1032,8 @@ out:
 	}
 
 	gdk_drag_status (context, action, time);
+
+	return TRUE;
 }
 
 static void
@@ -1186,18 +1143,18 @@ slot_proxy_handle_drop (GtkWidget                *widget,
 			nautilus_view_drop_proxy_received_uris (target_view,
 								uri_list,
 								target_uri,
-								context->action);
-			eel_g_list_free_deep (uri_list);
+								gdk_drag_context_get_selected_action (context));
+			g_list_free_full (uri_list, g_free);
 		} else if (drag_info->info == NAUTILUS_ICON_DND_URI_LIST) {
 			nautilus_view_drop_proxy_received_uris (target_view,
 								drag_info->data.uri_list,
 								target_uri,
-								context->action);
+								gdk_drag_context_get_selected_action (context));
 		} if (drag_info->info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
 			nautilus_view_drop_proxy_received_netscape_url (target_view,
 									drag_info->data.netscape_url,
 									target_uri,
-									context->action);
+									gdk_drag_context_get_selected_action (context));
 		}
 
 
@@ -1235,7 +1192,7 @@ slot_proxy_drag_data_received (GtkWidget          *widget,
 	drag_info->have_data = TRUE;
 	drag_info->info = info;
 
-	if (data->length < 0) {
+	if (gtk_selection_data_get_length (data) < 0) {
 		drag_info->have_valid_data = FALSE;
 		return;
 	}
@@ -1251,7 +1208,7 @@ slot_proxy_drag_data_received (GtkWidget          *widget,
 
 		drag_info->have_valid_data = drag_info->data.uri_list != NULL;
 	} else if (info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
-		drag_info->data.netscape_url = g_strdup ((char *) data->data);
+		drag_info->data.netscape_url = g_strdup ((char *) gtk_selection_data_get_data (data));
 
 		drag_info->have_valid_data = drag_info->data.netscape_url != NULL;
 	}
