@@ -29,8 +29,6 @@
 #include <libegg/eggtreemultidnd.h>
 
 #include <string.h>
-#include <eel/eel-gtk-macros.h>
-#include <eel/eel-glib-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -75,6 +73,8 @@ struct FMListModelDetails {
 	int drag_begin_y;
 
 	GPtrArray *columns;
+
+	GList *highlight_files;
 };
 
 typedef struct {
@@ -154,13 +154,6 @@ fm_list_model_get_column_type (GtkTreeModel *tree_model, int index)
 	case FM_LIST_MODEL_LARGE_ICON_COLUMN:
 	case FM_LIST_MODEL_LARGER_ICON_COLUMN:
 	case FM_LIST_MODEL_LARGEST_ICON_COLUMN:
-	case FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_SMALL_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGE_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGER_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN:
 		return GDK_TYPE_PIXBUF;
 	case FM_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN:
 		return G_TYPE_BOOLEAN;
@@ -253,11 +246,13 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	FileEntry *file_entry;
 	NautilusFile *file;
 	char *str;
-	GdkPixbuf *icon;
+	GdkPixbuf *icon, *rendered_icon;
+	GIcon *gicon, *emblemed_icon, *emblem_icon;
+	NautilusIconInfo *icon_info;
+	GEmblem *emblem;
+	GList *emblem_icons, *l;
 	int icon_size;
-	guint emblem_size;
 	NautilusZoomLevel zoom_level;
-	GList *emblem_pixbufs;
 	NautilusFile *parent_file;
 	char *emblems_to_ignore[3];
 	int i;
@@ -296,7 +291,8 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 
 			flags = NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS |
-				NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE;
+				NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE |
+				NAUTILUS_FILE_ICON_FLAGS_USE_MOUNT_ICON_AS_EMBLEM;
 			if (model->details->drag_view != NULL) {
 				GtkTreePath *path_a, *path_b;
 				
@@ -315,22 +311,9 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 				}
 			}
 
-			icon = nautilus_file_get_icon_pixbuf (file, icon_size, TRUE, flags);
-  
-			g_value_set_object (value, icon);
-			g_object_unref (icon);
-		}
-		break;
-	case FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_SMALL_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGE_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGER_EMBLEM_COLUMN:
-	case FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
+			gicon = nautilus_file_get_gicon (file, flags);
 
-		if (file != NULL) {
+			/* render emblems with GEmblemedIcon */
 			parent_file = nautilus_file_get_parent (file);
 			i = 0;
 			emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_TRASH;
@@ -341,21 +324,47 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 				nautilus_file_unref (parent_file);
 			}
 			emblems_to_ignore[i++] = NULL;
-			
-			zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
-			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
-			emblem_size = nautilus_icon_get_emblem_size_for_icon_size (icon_size);
-			if (emblem_size != 0) {
-				emblem_pixbufs = nautilus_file_get_emblem_pixbufs (file,
-										   emblem_size,
-										   TRUE,
-										   emblems_to_ignore);
-				if (emblem_pixbufs != NULL) {
-					icon = emblem_pixbufs->data;
-					g_value_set_object (value, icon);
+
+			emblem_icons = nautilus_file_get_emblem_icons (file,
+								       emblems_to_ignore);
+
+			/* pick only the first emblem we can render for the list view */
+			for (l = emblem_icons; l != NULL; l = l->next) {
+				emblem_icon = l->data;
+				if (nautilus_icon_theme_can_render (G_THEMED_ICON (emblem_icon))) {
+					emblem = g_emblem_new (emblem_icon);
+					emblemed_icon = g_emblemed_icon_new (gicon, emblem);
+
+					g_object_unref (gicon);
+					g_object_unref (emblem);
+					gicon = emblemed_icon;
+
+					break;
 				}
-				eel_gdk_pixbuf_list_free (emblem_pixbufs);
 			}
+
+			g_list_free_full (emblem_icons, g_object_unref);
+
+			icon_info = nautilus_icon_info_lookup (gicon, icon_size);
+			icon = nautilus_icon_info_get_pixbuf_at_size (icon_info, icon_size);
+
+			g_object_unref (icon_info);
+			g_object_unref (gicon);
+
+			if (model->details->highlight_files != NULL &&
+			    g_list_find_custom (model->details->highlight_files,
+			                        file, (GCompareFunc) nautilus_file_compare_location))
+			{
+				rendered_icon = eel_gdk_pixbuf_render (icon, 1, 255, 255, 0, 0);
+
+				if (rendered_icon != NULL) {
+					g_object_unref (icon);
+					icon = rendered_icon;
+				}
+			}
+
+			g_value_set_object (value, icon);
+			g_object_unref (icon);
 		}
 		break;
 	case FM_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN:
@@ -605,7 +614,7 @@ fm_list_model_get_first_iter_for_file (FMListModel          *model,
 		res = TRUE;
 		*iter = *(GtkTreeIter *)list->data;
 	}
-	eel_g_list_free_deep (list);
+	g_list_free_full (list, g_free);
 	
 	return res;
 }
@@ -868,7 +877,7 @@ fm_list_model_multi_drag_data_get (EggTreeMultiDragSource *drag_source,
 	}
 
 	if (gtk_target_list_find (drag_target_list,
-				  selection_data->target,
+				  gtk_selection_data_get_target (selection_data),
 				  &target_info)) {
 		nautilus_drag_drag_data_get (NULL,
 					     NULL,
@@ -1406,52 +1415,6 @@ fm_list_model_get_column_id_from_zoom_level (NautilusZoomLevel zoom_level)
 	g_return_val_if_reached (FM_LIST_MODEL_STANDARD_ICON_COLUMN);
 }
 
-NautilusZoomLevel
-fm_list_model_get_zoom_level_from_emblem_column_id (int column)
-{
-	switch (column) {
-	case FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_SMALLEST;
-	case FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_SMALLER;
-	case FM_LIST_MODEL_SMALL_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_SMALL;
-	case FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_STANDARD;
-	case FM_LIST_MODEL_LARGE_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_LARGE;
-	case FM_LIST_MODEL_LARGER_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_LARGER;
-	case FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN:
-		return NAUTILUS_ZOOM_LEVEL_LARGEST;
-	}
-
-	g_return_val_if_reached (NAUTILUS_ZOOM_LEVEL_STANDARD);
-}
-
-int
-fm_list_model_get_emblem_column_id_from_zoom_level (NautilusZoomLevel zoom_level)
-{
-	switch (zoom_level) {
-	case NAUTILUS_ZOOM_LEVEL_SMALLEST:
-		return FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_SMALLER:
-		return FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_SMALL:
-		return FM_LIST_MODEL_SMALL_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_STANDARD:
-		return FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_LARGE:
-		return FM_LIST_MODEL_LARGE_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_LARGER:
-		return FM_LIST_MODEL_LARGER_EMBLEM_COLUMN;
-	case NAUTILUS_ZOOM_LEVEL_LARGEST:
-		return FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN;
-	}
-
-	g_return_val_if_reached (FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN);
-}
-
 void
 fm_list_model_set_drag_view (FMListModel *model,
 			     GtkTreeView *view,
@@ -1552,6 +1515,11 @@ fm_list_model_finalize (GObject *object)
 
 	model = FM_LIST_MODEL (object);
 
+	if (model->details->highlight_files != NULL) {
+		nautilus_file_list_free (model->details->highlight_files);
+		model->details->highlight_files = NULL;
+	}
+
 	g_free (model->details);
 
 	G_OBJECT_CLASS (fm_list_model_parent_class)->finalize (object);
@@ -1667,3 +1635,45 @@ fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *
 		}
 	}
 }
+
+static void
+refresh_row (gpointer data,
+             gpointer user_data)
+{
+	NautilusFile *file;
+	FMListModel *model;
+	GList *iters, *l;
+	GtkTreePath *path;
+
+	model = user_data;
+	file = data;
+
+	iters = fm_list_model_get_all_iters_for_file (model, file);
+	for (l = iters; l != NULL; l = l->next) {
+		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), l->data);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, l->data);
+
+		gtk_tree_path_free (path);
+	}
+
+	g_list_free_full (iters, g_free);
+}
+
+void
+fm_list_model_set_highlight_for_files (FMListModel *model,
+                                       GList *files)
+{
+	if (model->details->highlight_files != NULL) {
+		g_list_foreach (model->details->highlight_files,
+		                refresh_row, model);
+		nautilus_file_list_free (model->details->highlight_files);
+		model->details->highlight_files = NULL;
+	}
+
+	if (files != NULL) {
+		model->details->highlight_files = nautilus_file_list_copy (files);
+		g_list_foreach (model->details->highlight_files,
+		                refresh_row, model);
+
+	}
+}
