@@ -29,7 +29,6 @@
 #include <config.h>
 #include "fm-tree-model.h"
 
-#include <eel/eel-glib-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-directory.h>
@@ -65,7 +64,6 @@ struct TreeNode {
 	GMount *mount;
 	GdkPixbuf *closed_pixbuf;
 	GdkPixbuf *open_pixbuf;
-	GdkPixbuf *emblem_pixbuf;
 
 	FMTreeModelRoot *root;
 
@@ -98,7 +96,6 @@ struct FMTreeModelDetails {
 	guint monitoring_update_idle_id;
 
 	gboolean show_hidden_files;
-	gboolean show_backup_files;
 	gboolean show_only_directories;
 
 	GList *highlighted_files;
@@ -211,7 +208,6 @@ tree_node_destroy (FMTreeModel *model, TreeNode *node)
 	object_unref_if_not_NULL (node->icon);
 	object_unref_if_not_NULL (node->closed_pixbuf);
 	object_unref_if_not_NULL (node->open_pixbuf);
-	object_unref_if_not_NULL (node->emblem_pixbuf);
 
 	g_assert (node->done_loading_id == 0);
 	g_assert (node->files_added_id == 0);
@@ -267,17 +263,58 @@ get_menu_icon_for_file (TreeNode *node,
 			NautilusFileIconFlags flags)
 {
 	NautilusIconInfo *info;
+	GIcon *gicon, *emblem_icon, *emblemed_icon;
+	GEmblem *emblem;
 	GdkPixbuf *pixbuf, *retval;
 	gboolean highlight;
 	int size;
 	FMTreeModel *model;
+	GList *emblem_icons, *l;
+	char *emblems_to_ignore[3];
+	int i;
 
 	size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
 
-	info = nautilus_file_get_icon (file, size, flags);
+	gicon = nautilus_file_get_gicon (file, flags);
+
+	i = 0;
+	emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_TRASH;
+
+	if (node->parent && node->parent->file) {
+		if (!nautilus_file_can_write (node->parent->file)) {
+			emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE;
+		}
+	}
+	
+	emblems_to_ignore[i++] = NULL;
+
+	emblem = NULL;
+	emblem_icons = nautilus_file_get_emblem_icons (node->file,
+						       emblems_to_ignore);
+
+	/* pick only the first emblem we can render for the tree view */
+	for (l = emblem_icons; l != NULL; l = l->next) {
+		emblem_icon = l->data;
+		if (nautilus_icon_theme_can_render (G_THEMED_ICON (emblem_icon))) {
+			emblem = g_emblem_new (emblem_icon);
+			emblemed_icon = g_emblemed_icon_new (gicon, emblem);
+
+			g_object_unref (gicon);
+			g_object_unref (emblem);
+			gicon = emblemed_icon;
+
+			break;
+		}
+	}
+
+	g_list_free_full (emblem_icons, g_object_unref);
+
+	info = nautilus_icon_info_lookup (gicon, size);
 	retval = nautilus_icon_info_get_pixbuf_nodefault_at_size (info, size);
 	model = node->root->model;
 
+	g_object_unref (gicon);
+
 	highlight = (g_list_find_custom (model->details->highlighted_files,
 	                                 file, (GCompareFunc) nautilus_file_compare_location) != NULL);
 
@@ -337,60 +374,6 @@ tree_node_update_open_pixbuf (TreeNode *node)
 	return tree_node_update_pixbuf (node, &node->open_pixbuf, NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER);
 }
 
-static GdkPixbuf *
-tree_node_get_emblem_pixbuf_internal (TreeNode *node)
-{
-	GdkPixbuf *pixbuf;
-	GList *emblem_pixbufs;
-	char *emblems_to_ignore[3];
-	int i;
-	
-	i = 0;
-	emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_TRASH;
-	
-	if (node->parent && node->parent->file) {
-		if (!nautilus_file_can_write (node->parent->file)) {
-			emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE;
-		}
-	}
-	
-	emblems_to_ignore[i++] = NULL;
-
-	emblem_pixbufs = nautilus_file_get_emblem_pixbufs (node->file,
-							   nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU),
-							   TRUE,
-							   emblems_to_ignore);
-	
-
-	if (emblem_pixbufs != NULL) {
-		pixbuf = g_object_ref (emblem_pixbufs->data);
-	} else {
-		pixbuf = NULL;
-	}
-
-	eel_gdk_pixbuf_list_free (emblem_pixbufs);
-
-	return pixbuf;
-}
-
-static gboolean
-tree_node_update_emblem_pixbuf (TreeNode *node)
-{
-	GdkPixbuf *pixbuf;
-
-	if (node->emblem_pixbuf == NULL) {
-		return FALSE;
-	}
-	pixbuf = tree_node_get_emblem_pixbuf_internal (node);
-	if (pixbuf == node->emblem_pixbuf) {
-		g_object_unref (pixbuf);
-		return FALSE;
-	}
-	g_object_unref (node->emblem_pixbuf);
-	node->emblem_pixbuf = pixbuf;
-	return TRUE;
-}
-
 static gboolean
 tree_node_update_display_name (TreeNode *node)
 {
@@ -431,15 +414,6 @@ tree_node_get_open_pixbuf (TreeNode *node)
 	return node->open_pixbuf;
 }
 
-static GdkPixbuf *
-tree_node_get_emblem_pixbuf (TreeNode *node)
-{
-	if (node->emblem_pixbuf == NULL) {
-		node->emblem_pixbuf = tree_node_get_emblem_pixbuf_internal (node);
-	}
-	return node->emblem_pixbuf;
-}
-
 static const char *
 tree_node_get_display_name (TreeNode *node)
 {
@@ -858,7 +832,6 @@ update_node_without_reporting (FMTreeModel *model, TreeNode *node)
 	changed |= tree_node_update_display_name (node);
 	changed |= tree_node_update_closed_pixbuf (node);
 	changed |= tree_node_update_open_pixbuf (node);
-	changed |= tree_node_update_emblem_pixbuf (node);
 
 	return changed;
 }
@@ -923,7 +896,6 @@ should_show_file (FMTreeModel *model, NautilusFile *file)
 
 	should = nautilus_file_should_show (file,
 					    model->details->show_hidden_files,
-					    model->details->show_backup_files,
 					    TRUE);
 
 	if (should
@@ -1128,11 +1100,10 @@ start_monitoring_directory (FMTreeModel *model, TreeNode *node)
 		 G_CALLBACK (files_changed_callback), node->root);
 
 	set_done_loading (model, node, nautilus_directory_are_all_files_seen (directory));
-	
+
 	attributes = get_tree_monitor_attributes ();
 	nautilus_directory_file_monitor_add (directory, model,
 					     model->details->show_hidden_files,
-					     model->details->show_backup_files,
 					     attributes, files_changed_callback, node->root);
 }
 
@@ -1152,8 +1123,6 @@ fm_tree_model_get_column_type (GtkTreeModel *model, int index)
 		return GDK_TYPE_PIXBUF;
 	case FM_TREE_MODEL_OPEN_PIXBUF_COLUMN:
 		return GDK_TYPE_PIXBUF;
-	case FM_TREE_MODEL_EMBLEM_PIXBUF_COLUMN:
-		return GDK_TYPE_PIXBUF;
 	case FM_TREE_MODEL_FONT_STYLE_COLUMN:
 		return PANGO_TYPE_STYLE;
 	default:
@@ -1297,10 +1266,6 @@ fm_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int column, GVa
 		g_value_init (value, GDK_TYPE_PIXBUF);
 		g_value_set_object (value, node == NULL ? NULL : tree_node_get_open_pixbuf (node));
 		break;
-	case FM_TREE_MODEL_EMBLEM_PIXBUF_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
-		g_value_set_object (value, node == NULL ? NULL : tree_node_get_emblem_pixbuf (node));
-		break;
 	case FM_TREE_MODEL_FONT_STYLE_COLUMN:
 		g_value_init (value, PANGO_TYPE_STYLE);
 		if (node == NULL) {
@@ -1718,7 +1683,6 @@ fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 		return;
 	}
 	model->details->show_hidden_files = show_hidden_files;
-	model->details->show_backup_files = show_hidden_files;
 	stop_monitoring (model);
 	if (!show_hidden_files) {
 		destroy_by_function (model, nautilus_file_is_hidden_file);
