@@ -20,10 +20,6 @@
 
 #include <config.h>
 #include <string.h>
-#include <eel/eel-debug.h>
-#include <eel/eel-gtk-extensions.h>
-#include <eel/eel-glib-extensions.h>
-#include <eel/eel-string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
@@ -32,7 +28,6 @@
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
-#include <libnautilus-private/nautilus-marshal.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-icon-dnd.h>
 #include "nautilus-pathbar.h"
@@ -98,8 +93,12 @@ G_DEFINE_TYPE (NautilusPathBar,
 
 static void     nautilus_path_bar_finalize                 (GObject         *object);
 static void     nautilus_path_bar_dispose                  (GObject         *object);
-static void     nautilus_path_bar_size_request             (GtkWidget       *widget,
-							    GtkRequisition  *requisition);
+static void     nautilus_path_bar_get_preferred_width      (GtkWidget        *widget,
+							    gint             *minimum,
+							    gint             *natural);
+static void     nautilus_path_bar_get_preferred_height     (GtkWidget        *widget,
+							    gint             *minimum,
+							    gint             *natural);
 static void     nautilus_path_bar_unmap                    (GtkWidget       *widget);
 static void     nautilus_path_bar_size_allocate            (GtkWidget       *widget,
 							    GtkAllocation   *allocation);
@@ -349,19 +348,18 @@ static void
 nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
 {
         GObjectClass *gobject_class;
-        GtkObjectClass *object_class;
         GtkWidgetClass *widget_class;
         GtkContainerClass *container_class;
 
         gobject_class = (GObjectClass *) path_bar_class;
-        object_class = (GtkObjectClass *) path_bar_class;
         widget_class = (GtkWidgetClass *) path_bar_class;
         container_class = (GtkContainerClass *) path_bar_class;
 
         gobject_class->finalize = nautilus_path_bar_finalize;
         gobject_class->dispose = nautilus_path_bar_dispose;
 
-        widget_class->size_request = nautilus_path_bar_size_request;
+	widget_class->get_preferred_height = nautilus_path_bar_get_preferred_height;
+	widget_class->get_preferred_width = nautilus_path_bar_get_preferred_width;
 	widget_class->unmap = nautilus_path_bar_unmap;
         widget_class->size_allocate = nautilus_path_bar_size_allocate;
         widget_class->style_set = nautilus_path_bar_style_set;
@@ -376,7 +374,7 @@ nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
 
         path_bar_signals [PATH_CLICKED] =
                 g_signal_new ("path-clicked",
-		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_OBJECT_CLASS_TYPE (path_bar_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (NautilusPathBarClass, path_clicked),
 		  NULL, NULL,
@@ -385,13 +383,15 @@ nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
 		  G_TYPE_FILE);
 	 path_bar_signals [PATH_SET] =
 		g_signal_new ("path-set",
-		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_OBJECT_CLASS_TYPE (path_bar_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (NautilusPathBarClass, path_set),
 		  NULL, NULL,
 		  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1,
 		  G_TYPE_FILE);
+
+	 gtk_container_class_handle_border_width (container_class);
 }
 
 
@@ -461,45 +461,71 @@ nautilus_path_bar_dispose (GObject *object)
  * available space.
  */
 static void
-nautilus_path_bar_size_request (GtkWidget      *widget,
-			        GtkRequisition *requisition)
+nautilus_path_bar_get_preferred_width (GtkWidget *widget,
+				       gint      *minimum,
+				       gint      *natural)
 {
-        ButtonData *button_data;
-        NautilusPathBar *path_bar;
-        GtkRequisition child_requisition;
-        GList *list;
-	guint border_width;
+	ButtonData *button_data;
+	NautilusPathBar *path_bar;
+	GList *list;
+	gint child_height;
+	gint height;
+	gint child_min, child_nat;
 
-        path_bar = NAUTILUS_PATH_BAR (widget);
+	path_bar = NAUTILUS_PATH_BAR (widget);
 
-        requisition->width = 0;
-        requisition->height = 0;
+	*minimum = *natural = 0;
+	height = 0;
 
 	for (list = path_bar->button_list; list; list = list->next) {
 		button_data = BUTTON_DATA (list->data);
-                gtk_widget_size_request (button_data->button, &child_requisition);
-                requisition->width = MAX (child_requisition.width, requisition->width);
-                requisition->height = MAX (child_requisition.height, requisition->height);
-        }
+		gtk_widget_get_preferred_width (button_data->button, &child_min, &child_nat);
+		gtk_widget_get_preferred_height (button_data->button, &child_height, NULL);
+		height = MAX (height, child_height);
 
-        /* Add space for slider, if we have more than one path */
-        /* Theoretically, the slider could be bigger than the other button.  But we're */
-        /* not going to worry about that now.*/
+		if (button_data->type == NORMAL_BUTTON) {
+			/* Use 2*Height as button width because of ellipsized label.  */
+			child_min = MAX (child_min, child_height * 2);
+			child_nat = MAX (child_min, child_height * 2);
+		}
 
-        path_bar->slider_width = MIN(requisition->height * 2 / 3 + 5, requisition->height);
-	if (path_bar->button_list && path_bar->button_list->next != NULL) {
-		requisition->width += (path_bar->spacing + path_bar->slider_width) * 2;
+		*minimum = MAX (*minimum, child_min);
+		*natural = MAX (*natural, child_nat);
 	}
 
-        gtk_widget_size_request (path_bar->up_slider_button, &child_requisition);
-        gtk_widget_size_request (path_bar->down_slider_button, &child_requisition);
+	/* Add space for slider, if we have more than one path */
+	/* Theoretically, the slider could be bigger than the other button.  But we're
+	 * not going to worry about that now.
+	 */
+	path_bar->slider_width = MIN (height * 2 / 3 + 5, height);
 
-	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-        requisition->width += border_width * 2;
-        requisition->height += border_width * 2;
+	if (path_bar->button_list && path_bar->button_list->next != NULL) {
+		*minimum += (path_bar->spacing + path_bar->slider_width) * 2;
+		*natural += (path_bar->spacing + path_bar->slider_width) * 2;
+	}
+}
 
-	gtk_widget_set_size_request (widget, requisition->width,
-				     requisition->height);
+static void
+nautilus_path_bar_get_preferred_height (GtkWidget *widget,
+					gint      *minimum,
+					gint      *natural)
+{
+	ButtonData *button_data;
+	NautilusPathBar *path_bar;
+	GList *list;
+	gint child_min, child_nat;
+
+	path_bar = NAUTILUS_PATH_BAR (widget);
+
+	*minimum = *natural = 0;
+
+	for (list = path_bar->button_list; list; list = list->next) {
+		button_data = BUTTON_DATA (list->data);
+		gtk_widget_get_preferred_height (button_data->button, &child_min, &child_nat);
+
+		*minimum = MAX (*minimum, child_min);
+		*natural = MAX (*natural, child_nat);
+	}
 }
 
 static void
@@ -544,7 +570,6 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
         GList *list, *first_button;
         gint width;
         gint allocation_width;
-        gint border_width;
         gboolean need_sliders;
         gint up_slider_offset;
         gint down_slider_offset;
@@ -563,8 +588,7 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
                 return;
 	}
         direction = gtk_widget_get_direction (widget);
-        border_width = (gint) gtk_container_get_border_width (GTK_CONTAINER (path_bar));
-        allocation_width = allocation->width - 2 * border_width;
+        allocation_width = allocation->width;
 
   	/* First, we check to see if we need the scrollbars. */
   	if (path_bar->fake_root) {
@@ -573,12 +597,14 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
 		width = 0;
 	}
 
-	gtk_widget_get_child_requisition (BUTTON_DATA (path_bar->button_list->data)->button, &child_requisition);
+	gtk_widget_get_preferred_size (BUTTON_DATA (path_bar->button_list->data)->button,
+				       &child_requisition, NULL);
 	width += child_requisition.width;
 
         for (list = path_bar->button_list->next; list; list = list->next) {
         	child = BUTTON_DATA (list->data)->button;
-		gtk_widget_get_child_requisition (child, &child_requisition);
+		gtk_widget_get_preferred_size (child,
+					       &child_requisition, NULL);
                 width += child_requisition.width + path_bar->spacing;
 
 		if (list == path_bar->fake_root) {
@@ -610,12 +636,14 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
        		* button, then count backwards.
        		*/
       		/* Count down the path chain towards the end. */
-		gtk_widget_get_child_requisition (BUTTON_DATA (first_button->data)->button, &child_requisition);
+		gtk_widget_get_preferred_size (BUTTON_DATA (first_button->data)->button,
+					       &child_requisition, NULL);
                 width = child_requisition.width;
                 list = first_button->prev;
                 while (list && !reached_end) {
 	  		child = BUTTON_DATA (list->data)->button;
-			gtk_widget_get_child_requisition (child, &child_requisition);
+			gtk_widget_get_preferred_size (child,
+						       &child_requisition, NULL);
 
 	  		if (width + child_requisition.width + path_bar->spacing + slider_space > allocation_width) {
 	    			reached_end = TRUE;
@@ -634,7 +662,8 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
 
                 while (first_button->next && ! reached_end) {
 	  		child = BUTTON_DATA (first_button->next->data)->button;
-			gtk_widget_get_child_requisition (child, &child_requisition);
+			gtk_widget_get_preferred_size (child,
+						       &child_requisition, NULL);
 
 	  		if (width + child_requisition.width + path_bar->spacing + slider_space > allocation_width) {
 	      			reached_end = TRUE;
@@ -649,26 +678,27 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
         }
 
         /* Now, we allocate space to the buttons */
-        child_allocation.y = allocation->y + border_width;
-        child_allocation.height = MAX (1, (gint) allocation->height - border_width * 2);
+        child_allocation.y = allocation->y;
+        child_allocation.height = allocation->height;
 
         if (direction == GTK_TEXT_DIR_RTL) {
-                child_allocation.x = allocation->x + allocation->width - border_width;
+                child_allocation.x = allocation->x + allocation->width;
                 if (need_sliders || path_bar->fake_root) {
 	  		child_allocation.x -= (path_bar->spacing + path_bar->slider_width);
-	  		up_slider_offset = allocation->width - border_width - path_bar->slider_width;
+	  		up_slider_offset = allocation->width - path_bar->slider_width;
 		}
         } else {
-                child_allocation.x = allocation->x + border_width;
+                child_allocation.x = allocation->x;
                 if (need_sliders || path_bar->fake_root) {
-	  		up_slider_offset = border_width;
+	  		up_slider_offset = 0;
 	  		child_allocation.x += (path_bar->spacing + path_bar->slider_width);
 		}
         }
 
         for (list = first_button; list; list = list->prev) {
                 child = BUTTON_DATA (list->data)->button;
-		gtk_widget_get_child_requisition (child, &child_requisition);
+		gtk_widget_get_preferred_size (child,
+					       &child_requisition, NULL);
 
 		gtk_widget_get_allocation (widget, &widget_allocation);
 
@@ -678,12 +708,12 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
 		}
                 /* Check to see if we've don't have any more space to allocate buttons */
                 if (need_sliders && direction == GTK_TEXT_DIR_RTL) {
-	  		if (child_allocation.x - path_bar->spacing - path_bar->slider_width < widget_allocation.x + border_width) {
+	  		if (child_allocation.x - path_bar->spacing - path_bar->slider_width < widget_allocation.x) {
 			    break;
 			}
 		} else {
 			if (need_sliders && direction == GTK_TEXT_DIR_LTR) {
-	  			if (child_allocation.x + child_allocation.width + path_bar->spacing + path_bar->slider_width > widget_allocation.x + border_width + allocation_width) {
+	  			if (child_allocation.x + child_allocation.width + path_bar->spacing + path_bar->slider_width > widget_allocation.x + allocation_width) {
 	    				break;	
 				}	
 			}
@@ -695,10 +725,10 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
                 if (direction == GTK_TEXT_DIR_RTL) {
 			child_allocation.x -= path_bar->spacing;
 	  		down_slider_offset = child_allocation.x - widget_allocation.x - path_bar->slider_width;
-	  		down_slider_offset = border_width;
+	  		down_slider_offset = 0;
 		} else {
 			down_slider_offset = child_allocation.x - widget_allocation.x;
-	  		down_slider_offset = allocation->width - border_width - path_bar->slider_width;
+	  		down_slider_offset = allocation->width - path_bar->slider_width;
 	  		child_allocation.x += child_allocation.width + path_bar->spacing;
 		}
         }
@@ -872,7 +902,6 @@ nautilus_path_bar_scroll_down (NautilusPathBar *path_bar)
         GList *up_button;
         gint space_available;
         gint space_needed;
-        gint border_width;
         GtkTextDirection direction;
 	GtkAllocation allocation, button_allocation, slider_allocation;
 
@@ -886,7 +915,6 @@ nautilus_path_bar_scroll_down (NautilusPathBar *path_bar)
 
         gtk_widget_queue_resize (GTK_WIDGET (path_bar));
 
-        border_width = gtk_container_get_border_width (GTK_CONTAINER (path_bar));
         direction = gtk_widget_get_direction (GTK_WIDGET (path_bar));
   
         /* We find the button at the 'down' end that we have to make */
@@ -918,7 +946,7 @@ nautilus_path_bar_scroll_down (NautilusPathBar *path_bar)
         if (direction == GTK_TEXT_DIR_RTL) {
                 space_available = slider_allocation.x - allocation.x;
 	} else {
-                space_available = (allocation.x + allocation.width - border_width) -
+                space_available = (allocation.x + allocation.width) -
                         (slider_allocation.x + slider_allocation.width);
 	}
 
@@ -1268,26 +1296,27 @@ get_dir_name (ButtonData *button_data)
  * or not the contents are bold
  */
 static void
-label_size_request_cb (GtkWidget       *widget,
-		       GtkRequisition  *requisition,
-		       ButtonData      *button_data)
+set_label_size_request (GtkWidget       *alignment,
+			ButtonData      *button_data)
 {
         const gchar *dir_name = get_dir_name (button_data);
         PangoLayout *layout;
-        gint bold_width, bold_height;
+        gint width, height, bold_width, bold_height;
         gchar *markup;
 	
 	layout = gtk_widget_create_pango_layout (button_data->label, dir_name);
-        pango_layout_get_pixel_size (layout, &requisition->width, &requisition->height);
+        pango_layout_get_pixel_size (layout, &width, &height);
   
         markup = g_markup_printf_escaped ("<b>%s</b>", dir_name);
         pango_layout_set_markup (layout, markup, -1);
         g_free (markup);
 
         pango_layout_get_pixel_size (layout, &bold_width, &bold_height);
-        requisition->width = MAX (requisition->width, bold_width);
-        requisition->height = MAX (requisition->height, bold_height);
-  
+
+	gtk_widget_set_size_request (alignment,
+				     MAX (width, bold_width),
+				     MAX (height, bold_height));
+	
         g_object_unref (layout);
 }
 
@@ -1422,7 +1451,7 @@ setup_file_path_mounted_mount (GFile *location, ButtonData *button_data)
 		g_object_unref (default_location);
 		g_object_unref (root);
 	}
-	eel_g_object_list_free (mounts);
+	g_list_free_full (mounts, g_object_unref);
 	return result;
 }
 
@@ -1510,7 +1539,7 @@ button_data_file_changed (NautilusFile *file,
 			  ButtonData *button_data)
 {
 	GFile *location, *current_location, *parent, *button_parent;
-	ButtonData *current_button_data, *temp_button_data;
+	ButtonData *current_button_data;
 	char *display_name;
 	NautilusPathBar *path_bar;
 	gboolean renamed, child;
@@ -1570,18 +1599,17 @@ button_data_file_changed (NautilusFile *file,
 			return;
 		}
 	} else if (nautilus_file_is_gone (file)) {
+		gint idx, position;
+
 		/* remove this and the following buttons */
-		GList *children;
+		position = g_list_position (path_bar->button_list,
+					    g_list_find (path_bar->button_list, button_data));
 
-		children = g_list_find (path_bar->button_list, button_data);
-
-		while (children != NULL) {
-			temp_button_data = children->data;
-
-			/* children are in reverse order */
-			children = children->prev;
-
-			gtk_container_remove (GTK_CONTAINER (path_bar), temp_button_data->button);
+		if (position != -1) {
+			for (idx = 0; idx <= position; idx++) {
+				gtk_container_remove (GTK_CONTAINER (path_bar),
+						      BUTTON_DATA (path_bar->button_list->data)->button);
+			}
 		}
 
 		g_object_unref (location);
@@ -1592,7 +1620,7 @@ button_data_file_changed (NautilusFile *file,
 	/* MOUNTs use the GMount as the name, so don't update for those */
 	if (button_data->type != MOUNT_BUTTON) {
 		display_name = nautilus_file_get_display_name (file);
-		if (eel_strcmp (display_name, button_data->dir_name) != 0) {
+		if (g_strcmp0 (display_name, button_data->dir_name) != 0) {
 			g_free (button_data->dir_name);
 			button_data->dir_name = g_strdup (display_name);
 		}
@@ -1659,16 +1687,6 @@ make_directory_button (NautilusPathBar  *path_bar,
 			button_data->is_base_dir = base_dir;
         }
 
-  	/* label_alignment is created because we can't override size-request
-   	* on label itself and still have the contents of the label centered
-   	* properly in the label's requisition
-   	*/
-
-        if (label_alignment) {
-                g_signal_connect (label_alignment, "size-request",
-			      	  G_CALLBACK (label_size_request_cb), button_data);
-	}
-	
 	if (button_data->path == NULL) {
         	button_data->path = g_object_ref (path);
 	}
@@ -1686,6 +1704,16 @@ make_directory_button (NautilusPathBar  *path_bar,
 	}
 
         button_data->file_is_hidden = file_is_hidden;
+
+	/* FIXME: Maybe we dont need this alignment at all and we can
+	 * use GtkMisc aligments or even GtkWidget:halign/valign center.
+	 *
+	 * The following function ensures that the alignment will always
+	 * request the same size whether the button's text is bold or not.
+	 */
+	if (label_alignment) {
+		set_label_size_request (label_alignment, button_data);
+	}
 			  
         gtk_container_add (GTK_CONTAINER (button_data->button), child);
         gtk_widget_show_all (button_data->button);
